% example run using
% - LS-APC-VB algorithm
% - LS-APC-G algorithm
% - optimization algorithm

clear all

%% set tuning parameters
x_errmax = 1e-2;
eps = 1; % smoothness parameter for optim

%% generate example data
p = 40; % measurements
n = 20; % time
x_true = [zeros(1,7) 0.4 0.7 0.9 1 1 1 zeros(1,3) 1 zeros(1,3)]'; % true source term

% generate example SRS matrix
M = randn(p,n);
M(M<0.0) = 0;

y = M*x_true; % formal
% generate random noise
noise_level = 0.5;
addnoise = noise_level*randn(size(y));
ind_yneg = find((y+addnoise)<0);
while ~isempty(ind_yneg)
    addnoise(ind_yneg) = noise_level*randn(size(ind_yneg));
    ind_yneg = find((y+addnoise)<0);
end
y = M*x_true + addnoise;


% prior source term
x_0 = zeros(size(x_true));


if 1 % scaling of data
    coef = max(abs(y));
    y = y./coef;
    M = M./coef;
end

%% run algorithms
%=== LS-APC-VB
[hat_x_lsapcvb,info] = alg_LSAPCVB_x0(y,M,100,x_errmax,x_0);

%=== LS-APC-G
it = 1e4; % number of Gibbs iterations
l_0 = -1; % prior l_0
like = 0; % compute likelihood
positive = 1; % x only positive
addpath ./Gibbs_related/
[MEAN,MED,MAXC,Ts] = Gibbs_LSAPC_x0_general(M,y',x_0,it,l_0,x_errmax,like,positive);
hat_x_lsapcg = MEAN.th;

%=== optimization algorithm
[hat_x_optim,info] = alg_optim_iterative(y,M,x_0,1/x_errmax,eps);

%% display results
figure;
plot(x_true,'red--','Linewidth',1)
hold on
plot(x_0,'black','Linewidth',1)
hold on
plot(hat_x_lsapcvb,'blue','Linewidth',2)
hold on
plot(hat_x_lsapcg,'green','Linewidth',2)
hold on
plot(hat_x_optim,'magenta','Linewidth',2)
hold off
legend('true source term','used prior source term','LS-APC-VB method','LS-APC-G method','optimization method')


